
#include <string>
#include <thread>

#include "card_person.h"
#include "loc_point.h"
#include "card_message_handle.h"
#include "area.h"
#include "mine.h"
#include "site_area.h"
#include "his_location.h"
#include "three_rates.h"
#include "select_tool.h"
#include "monkey_car/monkeycar_person.h"
#include "websocket/ws_common.h"

person::person(std::string type,uint32_t cardid,uint16_t needdisplay,int16_t t,int32_t deptid,int32_t level_id,uint32_t cid,int wl)
	:card_location_base(type,cardid,needdisplay,t,deptid,level_id,cid)
	,m_workLine(wl)
{
	m_message_handle.reset(new card_message_handle(this));
	m_his_location_card.reset(new location_staff(m_id,m_type));
}

person::~person()
{
}

void person::on_message(zloop<task*> * loop,const message_locinfo&loc,bool is_history)
{
	m_message_handle->on_message(loop,loc,is_history);
}

void person::clear()
{
	//m_site_area.reset(new site_area_hover);
	m_area_tool.reset(new area_tool);
	m_mine_tool.reset(new mine_tool);

	card_location_base::clear();
}

void person::site_hover(int sid)
{
	if(m_time<=0)
	{
		return;
	}
	log_info("__thread_id site_hover:%d,%x  [%x]",m_id,std::this_thread::get_id(),m_site_area.get());
	m_site_area->on_point(m_id,sid,nullptr, m_type);
}

std::shared_ptr<area_hover> person::get_area_hover()
{
	return m_area_tool->m_area_hover;
}

std::shared_ptr<mine_tool> person::get_mine_tool()
{
	return m_mine_tool;
}

std::shared_ptr<site_area_hover> person::get_site_area()
{
	return m_site_area;
}

void person::do_business(const point &pt,double acc)
{
	m_area_tool->on_point(m_id,pt,m_speed,m_type);
	m_site_area->on_point(m_id,0,this, m_type);
	handle_three_rates(pt);
}

void person::reset(std::shared_ptr<monkey_person> mp)
{
	m_monkeyPerson = mp;
}

void person::handle_three_rates(const point & pt)
{
	card_pos cp;
	cp.work_line=m_workLine;
	cp.biz_stat = get_stat();
	cp.x=pt.x;cp.y=pt.y;cp.z=pt.z;	
	const auto lm = m_area_tool->getLandmark();
	cp.enter_time = std::get<0>(lm)*1000;
	cp.area_id = std::get<3>(lm);
	put_three_rates(cp);
}

void person::on_timer()
{    
	if(!m_mine_tool->is_attendance())
		return;
	YA::_CARD_POS_ cp;

	uint64_t _time=0;
	point pt = getSmoothPoint(_time);
	const auto lm = m_area_tool->getLandmark();

	cp.enter_area_time = std::get<0>(lm)*1000;
	cp.rec_time = std::get<1>(lm);

	int32_t map_id = std::get<2>(lm);
	int32_t area_id = std::get<3>(lm);
	cp.map_id=map_id;cp.area_id=area_id;
	cp.landmark_id = std::get<4>(lm);
	cp.lm_direction = std::get<5>(lm);
	cp.landmark_dis=std::get<6>(lm);

	cp.biz_stat = get_stat();
	cp.down_time = m_mine_tool->get_down_time();
	cp.work_time = m_mine_tool->get_work_time();
	cp.is_on_duty= m_mine_tool->is_on_duty();
	log_info("on_timer here ...%d,%lld,%.2f,%.2f,%d,%d",m_id,_time,pt.x,pt.y,area_id,map_id);
	m_his_location_card->push(_time,pt,area_id,map_id);
	upt_card_pos(cp,pt);
}

point person::getSmoothPoint(uint64_t& t)
{
	point pt;
	loc_point lp = m_smo_tool->smooth_strategy();
	m_speed = lp.m_speed;
	m_stat  = lp.m_stat;
	pt.x = lp.x;
	pt.y = -lp.y;
	t=lp.m_time;
	if(auto  p = m_monkeyPerson.lock() )
	{
		if(p->is_on_bus())
		{
			m_stat = 7;
			pt = p->getPoint(t);	
			log_info("getpoint_oncar:%d,%lld",m_id,t);
		}
	}
	return pt;
}

