
#include <memory>
#include <unordered_map>
#include "log.h"

#include "db_api/CDBResultSet.h"
#include "db_api/CDBSingletonDefine.h"

#include "../card_base.h"

namespace db_card
{
	static std::string to13str(uint64_t data)
	{
		char ss[20]={0};
		sprintf(ss, "%013ld", data);

		return std::string(ss);
	}

	static uint64_t getId(uint32_t cardid,uint64_t type)
	{
	    return type<<32|cardid;
	}

	std::unordered_map<uint64_t,std::shared_ptr<card_location_base>> load_car(int64_t id64,const std::string&strategy)
	{
		std::string sql = "SELECT ve.vehicle_id, ve.card_id, c.card_type_id, \
						   ve.dept_id, ve.group_id, v.vehicle_type_id, vt.vehicle_level_id, \
						   vt.is_railroad AS vt_is_railroad,ve.need_display ,ve.power_alarm,\
						   vt.vehicle_category_id,v.bigger_car_flag,vc.over_speed \
						   FROM dat_vehicle_extend ve \
						   LEFT JOIN dat_vehicle v ON ve.vehicle_id = v.vehicle_id \
						   LEFT JOIN dat_card c ON ve.card_id = c.card_id \
						   LEFT JOIN dat_dept d ON ve.dept_id = d.dept_id \
						   LEFT JOIN dat_group g ON ve.group_id = g.group_id \
						   LEFT JOIN dat_vehicle_type vt ON v.vehicle_type_id = vt.vehicle_type_id \
						   LEFT JOIN dat_vehicle_category vc ON vc.vehicle_category_id = vt.vehicle_category_id \
						   WHERE c.state_id = 0";

		std::unordered_map<uint64_t,std::shared_ptr<card_location_base>> map;

		std::string card_id_str = to13str(id64);

		if(-1 == id64)
		{
			sql.append(";");
		}
		else
		{
			sql.append(" AND ve.card_id ='");
			sql.append(card_id_str);
			sql.append("';");

			log_info("基础数据 增加或修改车卡 sql=%s", sql.c_str());
		}

		std::string Error;
		YADB::CDBResultSet DBRes;
		sDBConnPool.Query(sql.c_str(),DBRes,Error);
		int nCount = DBRes.GetRecordCount( Error );
		if (nCount < 1)
		{
			log_error("增加或修改失败，数据库中找不到: card_id=%s", card_id_str.c_str());
			return map;
		}

		while ( DBRes.GetNextRecod(Error) )
		{
			unsigned int vehicle_id  = 0;
			DBRes.GetField( "vehicle_id",vehicle_id, Error );

			std::string card_id;
			DBRes.GetField( "card_id",card_id, Error );

			uint32_t vsid = atoi(card_id.substr(3).c_str());

			unsigned int card_type_id  = 0;
			DBRes.GetField( "card_type_id",card_type_id, Error );

			int dept_id = 0;
			DBRes.GetField( "dept_id",dept_id, Error );

			int group_id = 0;
			DBRes.GetField( "group_id",group_id, Error );

			int vehicle_type_id = 0;
			DBRes.GetField( "vehicle_type_id",vehicle_type_id, Error );

			int vehicle_level_id = 0;
			DBRes.GetField( "vehicle_level_id",vehicle_level_id, Error );

			int need_display = 0;
			DBRes.GetField( "need_display",need_display, Error );

			int power_alarm = 0;
			DBRes.GetField( "power_alarm",power_alarm, Error );

			int vehicle_category_id = 0;
			DBRes.GetField( "vehicle_category_id",vehicle_category_id, Error );

			int bigger_car_flag= 0;
			DBRes.GetField( "bigger_car_flag",bigger_car_flag, Error );

			double over_speed= 0;
			DBRes.GetField( "over_speed",over_speed, Error );

			auto clb = card_location_base::make_car(strategy,vsid,need_display,card_type_id,
					dept_id,vehicle_category_id, vehicle_type_id,vehicle_level_id,vehicle_id);
			uint64_t cardid = getId(vsid,card_type_id);
			log_info("cardId:%llu,id:%d dept_id:%d,need_display:%d-cardid:%s,categoryid:%d,vchile_id:%d,type:%d",
					cardid,vsid,dept_id,need_display,card_id.c_str(),vehicle_category_id,vehicle_id,card_type_id);
			map.insert({cardid,clb});

			log_info("基础数据 增加或修改车卡成功：卡id:%ld, dept_id:%d,need_display:%d,categoryid:%d",
					id64,dept_id,need_display,vehicle_category_id);
		}

		return map;
	}

	std::unordered_map<uint64_t,std::shared_ptr<card_location_base>> load_person(int64_t id64,const std::string&strategy)
	{
		std::string sql = "SELECT staff_id, s.card_id, c.card_type_id, s.dept_id, s.group_id, s.occupation_id, \
						   ol.occupation_level_id,s.worktype_id,s.need_display,s.work_line\
						   FROM dat_staff_extend s \
						   LEFT JOIN dat_card c ON s.card_id = c.card_id \
						   LEFT JOIN dat_occupation o ON s.occupation_id = o.occupation_id \
						   LEFT JOIN dat_occupation_level ol ON ol.occupation_level_id = o.occupation_level_id \
						   WHERE s.duty_id = 0 AND c.state_id = 0";

		std::unordered_map<uint64_t,std::shared_ptr<card_location_base>> map;
		std::string card_id_str = to13str(id64);
		if(-1 == id64)
		{
			sql.append(";");
		}
		else
		{
			sql.append(" AND s.card_id = '");
			sql.append(card_id_str);
			sql.append("';");

			log_info("基础数据 增加或修改人卡 sql=%s", sql.c_str());
		}

		std::string Error;
		YADB::CDBResultSet DBRes;
		sDBConnPool.Query(sql.c_str(),DBRes,Error);
		int nCount = DBRes.GetRecordCount( Error );
		if (nCount < 1)
		{
			log_error("增加或修改失败，数据库中找不到: card_id=%s", card_id_str.c_str());
			return map;
		}

		while ( DBRes.GetNextRecod(Error) )
		{
			unsigned int staff_id  = 0;
			DBRes.GetField( "staff_id",staff_id, Error );

			std::string card_id;
			DBRes.GetField( "card_id",card_id, Error );

			uint32_t vsid = atoi(card_id.substr(3).c_str());

			unsigned int card_type_id  = 0;
			DBRes.GetField( "card_type_id",card_type_id, Error );

			int dept_id = 0;
			DBRes.GetField( "dept_id",dept_id, Error );

			int group_id = 0;
			DBRes.GetField( "group_id",group_id, Error );

			int occupation_id = 0;
			DBRes.GetField( "occupation_id",occupation_id, Error );

			int occupation_level_id = 0;
			DBRes.GetField( "occupation_level_id",occupation_level_id, Error );

			int need_display = 0;
			DBRes.GetField( "need_display",need_display, Error );

			int work_line = 0;
			DBRes.GetField( "work_line",work_line, Error );
			//for now;
			//staff_id = vsid;

			std::shared_ptr<card_location_base> clb = card_location_base::make_person(strategy,vsid,need_display,card_type_id,
															dept_id,occupation_level_id,staff_id,work_line);
			uint64_t cardid = getId(vsid,card_type_id);
			log_info("cardId:%llu,id:%d dept_id:%d,need_display:%d,card:%s:work_line:%d,staff_id:%d,type:%d",
						cardid,vsid,dept_id,need_display,card_id.c_str(),work_line,staff_id,card_type_id);
			map.insert({cardid,clb});
		}

		return map;
	}

}
